'use strict';

const Promise = require("bluebird");
const streamToPromise = require("stream-to-promise");
const memoizee = require("memoizee");

const parseVolumeDescriptor = require("./parse/volume-descriptor");
const parsePathTable = require("./parse/path-table");

let imageOffset = 32 * 1024; // first 32KB are system area, ref http://wiki.osdev.org/ISO_9660#System_Area

module.exports = function createImage(seekable) {
    let image = {
        createReadStream: function createReadStream(start, end) {
            return Promise.try(() => {
                return seekable.createReadStream(start, end);
            });
        },
        readRange: function readRange(start, end) {
            return Promise.try(() => {
                return this.createReadStream(start, end);
            }).then((stream) => {
                return streamToPromise(stream);
            });
        },

        getPrimaryVolumeDescriptorBuffer: memoizee(function getPrimaryVolumeDescriptorBuffer() {
            return this.readRange(imageOffset, imageOffset + 2047);
        }),
        getPrimaryVolumeDescriptor: function getPrimaryVolumeDescriptor() {
            return Promise.try(() => {
                return this.getPrimaryVolumeDescriptorBuffer();
            }).then((buffer) => {
                return parseVolumeDescriptor(buffer);
            });
        },

        getSectorOffset: function getSectorOffset(sector) {
            return Promise.try(() => {
                return this.getPrimaryVolumeDescriptor();
            }).then((primaryVolumeDescriptor) => {
                return primaryVolumeDescriptor.data.sectorSize * sector;
            });
        },

        getPathTable: function getPathTable() {
            return Promise.try(() => {
                return this.getPrimaryVolumeDescriptor();
            }).then((primaryVolumeDescriptor) => {
                return Promise.try(() => {
                    let start = primaryVolumeDescriptor.data.pathTableLocationL * primaryVolumeDescriptor.data.sectorSize;
                    let end = start + primaryVolumeDescriptor.data.pathTableSize - 1;

                    return this.readRange(start, end);
                }).then((buffer) => {
                    return parsePathTable(buffer, primaryVolumeDescriptor.data.sectorSize);
                });
            });
        },
        getRootDirectory: function getRootDirectory() {
            return Promise.try(() => {
                return this.getPrimaryVolumeDescriptor();
            }).then((primaryVolumeDescriptor) => {
                return createDirectory(primaryVolumeDescriptor.data.rootDirectory);
            });
        }
    }

    const createDirectory = require("./object/directory")(image);

    return image;
}
